/***** traverseTree.c *****************************
 * Description: Read tree in Newick format and
 *   traverse it.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Sun Sep 16 20:28:48 2012
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "interface.h"
#include "eprintf.h"
#include "tree.h"
#include "traverse.h"

void scanFile(FILE *fp, Args *args);
void printHeader();

int main(int argc, char *argv[]){
  int i;
  char *version;
  Args *args;
  FILE *fp;

  version = "0.3";
  setprogname2("traverseTree");
  args = getArgs(argc, argv);
  if(args->v)
    printSplash(version);
  if(args->h || args->e)
    printUsage(version);
  if(args->numInputFiles == 0){
    fp = stdin;
    scanFile(fp, args);
  }else{
    for(i=0;i<args->numInputFiles;i++){
      fp = efopen(args->inputFiles[i],"r");
      scanFile(fp, args);
      fclose(fp);
    }
  }
  free(args);
  free(progname());
  return 0;
}

void scanFile(FILE *fp, Args *args){
  Node *root;

  setBisonFile(fp);
  while((root = parseTree()) != NULL){
    if(strcmp(args->t,"inorder") == 0){
      printf("Inorder Traversal\n");
      printHeader();
      printInorder(root);
    }if(strcmp(args->t,"preorder") == 0){
      printf("Preorder Traversal\n");
      printHeader();
      printPreorder(root);
    }if(strcmp(args->t,"postorder") == 0){
      printf("Postorder Traversal\n");
      printHeader();
      printPostorder(root);
    }
  }
}

void printHeader(){
  printf("ID\tLabel\tParent\tDist.\tType\n");
}
